% [data_process] = segmentation_SAXS(four_dat, data_process)
% This function is used to perform SAXS segmentation in case
% par.do_segmentation = 1 in the SAXS_caller_template
% inputs: 
% fnames: structure with the names of the paths used for saving the data
% par: parameters for plotting and scan (par.snake, par.fast_axis)
% data_process: symmetric intensity data averaged over the segments for the
% whole q range
% four_dat: result of fft from analyze_one_scan
% output:
% data_process : data after segmentation

%*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
%
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing la this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS scanning SAXS package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% Additionally, any publication using the package, or any translation of the 
%     code into another computing language should cite:
%    O. Bunk, M. Bech, T. H. Jensen, R. Feidenhans'l, T. Binderup, A. Menzel 
%    and F Pfeiffer, “Multimodal x-ray scatter imaging,” New J. Phys. 11,
%    123016 (2009). (doi: 10.1088/1367-2630/11/12/123016)
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [data_process] = segmentation_SAXS(four_dat, data_process)
import utils.adjust_projection

fnames = four_dat.fnames;
par = four_dat.par;

%%
%reshape the input
corrected_data = data_process.sym_data;
corrected_data = reshape(corrected_data,size(corrected_data,1)*size(corrected_data,2), []);
corrected_data(:,~any(corrected_data,1)) = [];
yet = corrected_data(:, par.pixel_range);
scan_dimension = [par.ind_max_x, par.ind_max_y];

%% step 2.2: correct and smooth the input data: save as an structure in data
data = struct();
% this might be readjusted
xq = logspace(1, log10(size(yet, 2)), round(size(yet, 2)/4));
for i = 1:size(yet,1)
    temp = (yet(i,:));
    temp = interp1(1:length(temp), temp, xq);
    temp_norm = median(temp);
    data(i).interp_data = temp./temp_norm;
    data(i).interp1_data = temp;
    data(i).smooth_data = (smooth(temp./temp_norm));
    
end


%% step 2.3 calculate the derivatives
for i = 1:length(data)
    %fprintf('point %d/%d \n', i, length(data))
    data(i).first_deriv = diff(data(i).interp_data);
    data(i).second_deriv = diff(diff(data(i).smooth_data));
end

%% step 2.4: find where the first and second derivative cross zero
zci = @(v) find(v(:).*circshift(v(:), [-1 0]) <= 0);
for i = 1:length(data)
    index_1 = zci(data(i).first_deriv);
    data(i).index_filter_1 = index_1(1:end-1);
    index_2 = zci(data(i).second_deriv);
    data(i).index_filter_2 = index_2(1:end-1);
    % fprintf('point %d/%d \n', i, length(data))
end

%% step 2.5: check the position of the index
do_it = 0;
if (do_it)
figure
    for i = 1:length(data)
        point = i;
        clf
        semilogy(data(point).interp_data, 'LineWidth', 3)
        hold on
        title(sprintf('point %d', point))
        plot(data(point).index_filter_1 + 1, 10 , '.k')
        plot(data(point).index_filter_2 + 2, 1 , '.r')
        %ylim([0.001 1000])
        pause(0.5)
    end
end
%% step 2.6: prepare pca table
% first derivative
for i = 1:length(data)
    index = data(i).index_filter_1;
    temp = zeros(size(data(i).first_deriv));
    temp(index) = data(i).interp_data(index);
    test = zeros(1,length(xq));
    test(index) = xq(index);
    data(i).q_1_pca = test;
    data(i).first_deriv_pca = temp;
end
% define the output intensities
first_pca = [data.first_deriv_pca];
first_pca = reshape(first_pca, length(data(1).first_deriv), length(data));
first_pca = transpose(first_pca);
first_pca(:,~any(first_pca,1)) = [];

% define the output scattering vectors
q_1_pca = [data.q_1_pca];
q_1_pca = reshape(q_1_pca, length(xq), length(data));
q_1_pca = transpose(q_1_pca);
q_1_pca(:,~any(q_1_pca,1)) = [];

%% second derivative
for i = 1:length(data)
    %fprintf('point %d/%d \n', i, length(data))
    index = data(i).index_filter_2;
    temp = zeros(size(data(i).second_deriv));
    temp(index) = data(i).interp_data(index);
    test = zeros(1,length(xq));
    test(index) = xq(index);
    data(i).q_2_pca = test;
    data(i).second_deriv_pca =temp;%.*xq(2:end);
end
% define the output intensities
second_pca = [data.second_deriv_pca];
second_pca = reshape(second_pca, length(data(1).second_deriv), length(data));
second_pca = transpose(second_pca);
second_pca(:,~any(second_pca,1)) = [];

% define the output scattering vectors
q_2_pca = [data.q_2_pca];
q_2_pca = reshape(q_2_pca, length(xq), length(data));
q_2_pca = transpose(q_2_pca);
q_2_pca(:,~any(q_2_pca,1)) = [];

%% define the input table for the pca
pca_input = [first_pca, q_1_pca, second_pca, q_2_pca];

%% step 2.7: calculate the PCA
[~,score,~,~,explained,~] = pca(pca_input,  'VariableWeights', 'variance', 'Centered', false);

%% step 2.8: define the best number of clusters (takes long time)
num_comp = length(find(explained > 0.35));
sel = score(:,1:num_comp);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% edit here:
distance = 'sqeuclidean';
range_clusters =  par.cluster_range;
% % % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%
if length(range_clusters) > 1
    rng(1);%for reproducibility
    tic
    eva = evalclusters(sel,'kmeans','silhouette', 'Klist', range_clusters, 'Distance',distance);
    toc
    clusters = eva.OptimalK;
    figure (185)
    plot(range_clusters, eva.CriterionValues)
    ylabel('Evaluation criterion [0 to 1]');
    xlabel('Number of clusters');
    grid on
else
    clusters = range_clusters;
end


% step 2.9: calculate the k-mean
opts = statset('Display','final');
rng(1);%for reproducibility

[idx,centroid] = kmeans(sel, clusters,'Distance',distance,...
    'Replicates',15,'MaxIter',100, 'Options',opts, 'Start', 'plus');

%% 2.12 plot the segregated image
fig = figure(211);
col = round(clusters/2);

%% Get positions to adjust projection based on snake_scan or fast_axis %%%
if isfield(four_dat,'original_pos_data')
    pos_data = four_dat.original_pos_data;
else
    pos_data = [];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i = 1:clusters
    subplot(col, 2,i)
    index = find(idx == i);
    im_cluster = ones((scan_dimension(2)+1)*(scan_dimension(1)+1), 1);
    im_cluster(index) = 0;
    im_cluster = reshape(im_cluster, scan_dimension(2)+1, scan_dimension(1)+1);

    im_cluster = adjust_projection(im_cluster, par.snake_scan, par.fast_axis_x, pos_data);

    im_cluster = bwareaopen(im_cluster, 20);
    
    imagesc(im_cluster);
    title(sprintf('Mask, cluster %d', i))
    colormap bone
    box on
    axis tight equal xy
    set(gca, 'FontSize', 12)
end
set(gcf,'color','w');
pause(0.01)
fprintf('saving data to %d_fourier.mat\n', fnames.first_scan_no);

save(sprintf('%sfourier_components/%s%s%d_fourier.mat', fnames.save_dir, ...
    fnames.sub_dir, fnames.rel_dir ,fnames.first_scan_no), 'four_dat',  'data_process', '-v6');
print(fig, sprintf('%sfigures_saxs/%s%s%d_clusters.jpg', fnames.save_dir, ...
        fnames.sub_dir, fnames.rel_dir ,fnames.first_scan_no),'-djpeg','-r300');
%clf(fig)
